@@ -18,7 +18,12 @@ def processing(Img1, Img2):
         [Img1, Img2] = masking(Img1, Img2)
 
     # Firs cross-correlation
-    [x1, y1, u1, v1] = corrFFT1(Img1, Img2)
+    if Parameters.direct_calc:
+        # direct cross correlation
+        [x1, y1, u1, v1] = corrDirect1(Img1, Img2)
+    else:
+        # FFT cros correlation
+        [x1, y1, u1, v1] = corrFFT1(Img1, Img2)
 
     # Iterate on the first grid if specified
     if Parameters.no_iter_1 > 1:
@@ -29,6 +34,7 @@ def processing(Img1, Img2):
 
     return x2, y2, u2, v2
 
+
 # FIRST STEP PIV CORRELATION FUNCTION
 def corrFFT1(Img1, Img2):
 
@@ -54,7 +60,6 @@ def corrFFT1(Img1, Img2):
     window_x = np.amin([2*np.round(window_x/2), box_size_x-4])
     window_y = np.amin([2*np.round(window_y/2), box_size_y-4])
 
-
     if Parameters.weighting:
         [i_matrix, j_matrix] = np.meshgrid(np.arange(box_size_x),
                 np.arange(box_size_y))
@@ -94,12 +99,12 @@ def corrFFT1(Img1, Img2):
                 SubImg2 = np.multiply(SubImg2, Weighting_Function)
 
             # Get the intensity of the images centered in the mean
-            SubImg1 = SubImg1-sum(sum(SubImg1))/(box_size_y*box_size_x)
-            SubImg2 = SubImg2-sum(sum(SubImg2))/(box_size_y*box_size_x)
+            SubImg1 = SubImg1-np.sum(SubImg1)/(box_size_y*box_size_x)
+            SubImg2 = SubImg2-np.sum(SubImg2)/(box_size_y*box_size_x)
 
             # Sum of intensity Image to Scale at end
-            Sigma1 = max(0.1, np.sqrt(sum(sum(SubImg1**2))))
-            Sigma2 = max(0.1, np.sqrt(sum(sum(SubImg2**2))))
+            Sigma1 = max(0.1, np.sqrt(np.sum(SubImg1**2)))
+            Sigma2 = max(0.1, np.sqrt(np.sum(SubImg2**2)))
 
             # Cross Correlation Function of the pair images scaled with
             # sigma1 and sigma2
@@ -109,7 +114,7 @@ def corrFFT1(Img1, Img2):
 
             # Find Peaks of the correlation Fucntion
             [epsilon_x,epsilon_y,col_idx,row_idx] = find_peaks(correlation,
-                    box_size_x, box_size_y, window_x, window_y)
+                    window_x, window_y)
 
             u_1[j,i] = epsilon_x + col_idx-box_size_x/2
             v_1[j,i] = epsilon_y + row_idx-box_size_y/2
@@ -181,8 +186,8 @@ def corrFFT1bis(Img1, Img2, x_1, y_1, u_1, v_1):
                     SubImg2 = np.multiply(SubImg2, Weighting_Function)
 
                 # Sum of intensity Image to Scale at end
-                Sigma1 = max(0.1,np.sqrt(sum(sum(SubImg1**2))))
-                Sigma2 = max(0.1,np.sqrt(sum(sum(SubImg2**2))))
+                Sigma1 = max(0.1, np.sqrt(np.sum(SubImg1**2)))
+                Sigma2 = max(0.1, np.sqrt(np.sum(SubImg2**2)))
 
                 # Cross Correlation Function of the pair images scaled with
                 # sigma1 and sigma2
@@ -192,8 +197,7 @@ def corrFFT1bis(Img1, Img2, x_1, y_1, u_1, v_1):
 
                 # Find Peaks of the correlation Function
                 epsilon_x, epsilon_y, col_idx, row_idx = find_peaks(
-                        correlation, box_size_x, box_size_y,
-                        window_x,window_y)
+                        correlation, window_x, window_y)
 
                 u_1[j,i] = (u_index[row_idx, col_idx] + epsilon_x +
                         col_idx-box_size_x/2)
@@ -290,8 +294,8 @@ def corrFFT2(Img1, Img2, x_1, y_1, u_1, v_1):
                     SubImg2 = np.multiply(SubImg2,Weighting_Function)
 
                 # Sum of intensity Image to Scale at end
-                Sigma1 = max(0.1,np.sqrt(sum(sum(SubImg1**2))))
-                Sigma2 = max(0.1,np.sqrt(sum(sum(SubImg2**2))))
+                Sigma1 = max(0.1, np.sqrt(np.sum(SubImg1**2)))
+                Sigma2 = max(0.1, np.sqrt(np.sum(SubImg2**2)))
 
                 # Cross Correlation Function of the pair images scaled with
                 # sigma1 and sigma2
@@ -301,8 +305,7 @@ def corrFFT2(Img1, Img2, x_1, y_1, u_1, v_1):
 
                 # Find Peaks of the correlation Fucntion
                 epsilon_x, epsilon_y, col_idx, row_idx = find_peaks(
-                        correlation, box_size_2_x, box_size_2_y,
-                        window_x, window_y)
+                        correlation, window_x, window_y)
 
                 u_2[j,i] = (u_index[row_idx, col_idx] + epsilon_x +
                         col_idx - box_size_2_x/2)
@@ -319,8 +322,94 @@ def corrFFT2(Img1, Img2, x_1, y_1, u_1, v_1):
 
     return x_2, y_2, u_2, v_2
 
+def corrDirect1(Img1, Img2):
+
+    # Definition of Parameters to reduce length
+    box_size_x = Parameters.box_size_1_x
+    box_size_y = Parameters.box_size_1_y
+    no_boxes_1_x = Parameters.no_boxes_1_x
+    no_boxes_1_y = Parameters.no_boxes_1_y
+    window_x = Parameters.window_x_1
+    window_y = Parameters.window_y_1
+
+    [Height,Width] = Img1.shape
+
+    # Initialize all matrix
+    box_origin_x_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    box_origin_y_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    x_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    y_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    u_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    v_1 = np.zeros([no_boxes_1_y, no_boxes_1_x])
+    correlation = np.zeros([window_y+1, window_x+1])
+
+    if Parameters.weighting:
+        [i_matrix, j_matrix] = np.meshgrid(np.arange(box_size_x),
+                np.arange(box_size_y))
+        Weighting_Function = weight_function(i_matrix, j_matrix,
+                box_size_x, box_size_y)
+
+    # Apply Gaussian filter to images only for first iteration
+    if Parameters.gaussian_filter:
+        [Img1, Img2] = gaussian_filter(Img1, Img2, Parameters.gaussian_filter)
+
+    for j in range(0, no_boxes_1_y):
+        for i in range(0, no_boxes_1_x):
+
+            x_1[j,i] = (1 + round((window_x/2+box_size_x)/2) +
+                round((i)*(Width-box_size_x-window_x/2-4)/(no_boxes_1_x-1)))
+            y_1[j,i] = (1 + round((window_y/2+box_size_y)/2) +
+                round((j)*(Height-box_size_y-window_y/2-4)/(no_boxes_1_y-1)))
+
+            box_origin_x_1[j,i] = x_1[j,i]+1-round(box_size_x/2)
+            box_origin_y_1[j,i] = y_1[j,i]+1-round(box_size_y/2)
+
+
+            for jj in range(-round(window_y/2), round(window_y/2)+1):
+                for ii in range(-round(window_x/2), round(window_x/2)+1):
+                    box_o_y = int(box_origin_y_1[j,i]+round(jj/2))
+                    box_o_x = int(box_origin_x_1[j,i]+round(ii/2))
+
+                    SubImg1 = (Img1[box_o_y-jj:box_o_y-jj+box_size_y,
+                            box_o_x-ii:box_o_x-ii+box_size_x])
+                    SubImg2 = (Img2[box_o_y:box_o_y+box_size_y,
+                            box_o_x:box_o_x+box_size_x])
+
+                    if Parameters.weighting==1:
+                        SubImg1 = np.multiply(SubImg1, Weighting_Function)
+                        SubImg2 = np.multiply(SubImg2, Weighting_Function)
+
+                    # Get the intensity of the images centered in the mean
+                    SubImg1 = SubImg1-np.sum(SubImg1)/(box_size_y*box_size_x)
+                    SubImg2 = SubImg2-np.sum(SubImg2)/(box_size_y*box_size_x)
+
+                    SubImg1 = np.divide(SubImg1, max(0.1,
+                        np.sqrt(np.sum(SubImg1**2))))
+                    SubImg2 = np.divide(SubImg2, max(0.1,
+                        np.sqrt(np.sum(SubImg2**2))))
+
+                    temp1 = jj+round(window_y/2)
+                    temp2 = ii+round(window_x/2)
+
+                    correlation[temp1, temp2] = (
+                        np.sum(np.multiply(SubImg1, SubImg2))
+                    )
+
+            # Find Peaks of the correlation Fucntion
+            [epsilon_x,epsilon_y,col_idx,row_idx] = find_peaks(correlation,
+                    window_x, window_y, 0)
+
+            u_1[j,i] = epsilon_x + col_idx - (window_x/2)
+            v_1[j,i] = epsilon_y + row_idx - (window_y/2)
+
+    return x_1, y_1, u_1, v_1, box_origin_x_1
+
+
+
 # FIND PEAKS POSITION ON CORRELATION FUNCTION
-def find_peaks(correlation, box_size_x, box_size_y, window_x, window_y):
+def find_peaks(correlation, window_x, window_y, westerweel = 1):
+
+    box_size_y, box_size_x = correlation.shape
 
     # Indices of searching windows
     ini_cor_y = int(np.round(box_size_y/2-window_y/2))
@@ -383,11 +472,11 @@ def find_peaks(correlation, box_size_x, box_size_y, window_x, window_y):
     [weight_i, weight_j] = np.meshgrid(np.arange(-1,2),np.arange(-1,2))
 
     # Bias correction based on westerweel
-    fit_peak = (np.divide(np.divide(
-        fit_peak,1-np.abs(max_row-box_size_y/2 + weight_j)/box_size_y),
-        1-np.abs(max_col-box_size_x/2+weight_i)/box_size_x))
-    fit_peak[np.where(fit_peak<0.001)] = 0.001
-
+    if westerweel:
+        fit_peak = (np.divide(np.divide(
+            fit_peak,1-np.abs(max_row-box_size_y/2 + weight_j)/box_size_y),
+            1-np.abs(max_col-box_size_x/2+weight_i)/box_size_x))
+        fit_peak[np.where(fit_peak<0.001)] = 0.001
 
     # Get sub-pixel accuracy with gaussian stimator
     if (fit_peak[1,1] == np.amin(fit_peak[1,:])):
@@ -864,9 +953,9 @@ def weight_function(i_matrix, j_matrix, box_size_x, box_size_y):
     zeta = i_matrix/box_size_x-0.5-0.5/box_size_x
     eta = j_matrix/box_size_y-0.5-0.5/box_size_y
     Weighting_Func = (9*np.multiply(4*np.power(zeta,2)-4*np.abs(zeta)+1,
-        4*np.power(zeta,2)-4*np.abs(eta)+1))
+        4*np.power(eta,2)-4*np.abs(eta)+1))
 
-    return Weighting_Funcfilter
+    return Weighting_Func
 
 def masking(Img1, Img2):
 
